@@ -1,9 +1,5 @@
-# python3
-
 """Generate cross references from a project."""
 
-from __future__ import print_function
-
 import collections
 import re
 from typing import Optional
@@ -101,7 +97,7 @@ class AttrError(Exception):
 
 
 @attr.s
-class PytypeValue(object):
+class PytypeValue:
   """Stores a value inferred by pytype."""
 
   module = attr.ib()
@@ -158,7 +154,7 @@ class PytypeValue(object):
 
 
 @attr.s
-class Module(object):
+class Module:
   """Module representation."""
   name = attr.ib()
 
@@ -171,7 +167,7 @@ class Module(object):
 
 
 @attr.s
-class DocString(object):
+class DocString:
   """Store the text and location of a docstring."""
 
   text = attr.ib()
@@ -198,7 +194,7 @@ class DocString(object):
 
 
 @attr.s
-class Definition(object):
+class Definition:
   """A symbol definition.
 
   Attributes:
@@ -260,7 +256,7 @@ class Definition(object):
 
 
 @attr.s
-class Remote(object):
+class Remote:
   """A symbol from another module."""
 
   module = attr.ib()
@@ -285,7 +281,7 @@ class Remote(object):
 
 
 @attr.s
-class DefLocation(object):
+class DefLocation:
   """A location of a symbol definition.
 
   Attributes:
@@ -301,7 +297,7 @@ class DefLocation(object):
 
 
 @attr.s
-class FunctionParam(object):
+class FunctionParam:
   """A link between a function def and the defs of its params."""
 
   def_id = attr.ib(type=str)
@@ -310,7 +306,7 @@ class FunctionParam(object):
 
 
 @attr.s
-class Reference(object):
+class Reference:
   """A symbol holding a reference to a definition.
 
   Attributes:
@@ -341,7 +337,7 @@ class Reference(object):
 
 
 @attr.s
-class NameArg(object):
+class NameArg:
   """Representation of a single-variable function call argument."""
 
   name = attr.ib()
@@ -349,7 +345,7 @@ class NameArg(object):
 
 
 @attr.s
-class ExprArg(object):
+class ExprArg:
   """Representation of an expression function call argument."""
 
   names = attr.ib()
@@ -357,7 +353,7 @@ class ExprArg(object):
 
 
 @attr.s
-class Funcall(object):
+class Funcall:
   """Representation of a function call."""
 
   name = attr.ib()
@@ -368,7 +364,7 @@ class Funcall(object):
   return_type = attr.ib()
 
 
-class Env(object):
+class Env:
   """A collection of namespaced symbols."""
 
   def __init__(self, ast, scope, parent, cls):
@@ -442,18 +438,6 @@ class Env(object):
 # Also names like visit_Name are self-documenting and do not need docstrings.
 
 
-class CollectNamesVisitor(ast_visitor.BaseVisitor):
-  """Collect all occurences of Name in a subtree."""
-
-  def __init__(self, ast):
-    super(CollectNamesVisitor, self).__init__(ast)
-    self.names = []
-
-  def visit_Name(self, node):
-    if isinstance(node.ctx, self._ast.Load):
-      self.names.append(node)
-
-
 class ScopedVisitor(ast_visitor.BaseVisitor):
   """An AST node visitor that keeps track of scopes and environments.
 
@@ -467,7 +451,7 @@ class ScopedVisitor(ast_visitor.BaseVisitor):
   # anything by way of maintainability or readability?
 
   def __init__(self, ast, module_name, **kwargs):
-    super(ScopedVisitor, self).__init__(ast=ast, **kwargs)  # pytype: disable=wrong-keyword-args
+    super().__init__(ast=ast, **kwargs)  # pytype: disable=wrong-keyword-args
     self.stack = []
     self.class_ids = []
     self.envs = {}
@@ -533,12 +517,12 @@ class ScopedVisitor(ast_visitor.BaseVisitor):
     self.add_scope(node)
 
   def enter_Module(self, node):
-    super(ScopedVisitor, self).enter_Module(node)  # pytype: disable=attribute-error
+    super().enter_Module(node)  # pytype: disable=attribute-error
     self.add_scope(node)
 
   def leave(self, node):
     """If the node has introduced a new scope, we need to pop it off."""
-    super(ScopedVisitor, self).leave(node)
+    super().leave(node)
     if node == self.stack[-1]:
       self.stack.pop()
 
@@ -547,9 +531,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
   """Visitor that generates indexes."""
 
   def __init__(self, ast, src, module_name):
-    super(IndexVisitor, self).__init__(ast=ast,
-                                       src_code=src,
-                                       module_name=module_name)
+    super().__init__(ast=ast, src_code=src, module_name=module_name)
     self.defs = {}
     self.locs = collections.defaultdict(list)
     self.refs = []
@@ -589,7 +571,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
     if isinstance(node, str):
       # We replace nodes with their names after visiting them.
       return node
-    return super(IndexVisitor, self)._get_node_name(node)
+    return super()._get_node_name(node)
 
   def make_def(self, node, **kwargs):
     """Make a definition from a node."""
@@ -717,7 +699,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
     defn = self.add_local_def(node, data=data,
                               doc=DocString.from_node(self._ast, node))
     self.classmap[d[0]] = defn
-    super(IndexVisitor, self).enter_ClassDef(node)
+    super().enter_ClassDef(node)
 
   def enter_FunctionDef(self, node):
     last_line = max(node.lineno, node.body[0].lineno - 1)
@@ -777,9 +759,8 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
 
   def visit_Call(self, node):
     name = self._get_node_name(node)
-    name_visitor = CollectNamesVisitor(self._ast)
-    name_visitor.visit(node)
-    arg_varnames = [x.id for x in name_visitor.names]
+    # We have replaced Name() in args with the corresponding string
+    arg_varnames = [x for x in node.args if isinstance(x, str)]
     seen = set()
     for _, (_, _, data) in self.match(node):
       call, return_type = data
@@ -803,7 +784,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
     # the location of the value on which the attribute is accessed, in order to
     # link function calls. We'll manually adjust the location later.
     for unused_loc, (op, unused_symbol, data) in self.match(node):
-      if op == "LOAD_ATTR":
+      if op in ("LOAD_ATTR", "LOAD_METHOD"):
         ref = self.add_local_ref(
             node,
             target=node.value,
@@ -845,14 +826,21 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
         # Shift symbol/loc back to the unaliased name.
         symbol = alias.name
         m = re.search("[ ,]" + symbol + r"\b", self.source.line(loc.line))
-        assert m is not None
+        if m is None:
+          # TODO(slebedev): Support multi-line from-imports.
+          continue
         c, _ = m.span()
         loc = source.Location(loc.line, c + 1)
 
       try:
         [imported] = _unwrap(data)
-      except ValueError:
-        continue  # Unresolved import.
+      except (TypeError, ValueError):
+        resolved = False
+      else:
+        resolved = not isinstance(imported, abstract.Unsolvable)
+
+      if not resolved:
+        continue
 
       if op == "STORE_NAME":
         # for |import x.y as z| or |from x import y as z| we want {z: x.y}
@@ -904,7 +892,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
 # pylint: enable=missing-docstring
 
 
-class Indexer(object):
+class Indexer:
   """Runs the indexer visitor and collects its results."""
 
   def __init__(self,
@@ -1007,8 +995,12 @@ class Indexer(object):
 
     try:
       [imported] = _unwrap(defn.data)
-    except ValueError:
-      # Unresolved module.
+    except (TypeError, ValueError):
+      resolved = False
+    else:
+      resolved = not isinstance(imported, abstract.Unsolvable)
+
+    if not resolved:
       return Remote(defn.name, name=attr_name, resolved=False)
 
     assert not isinstance(imported, abstract.Module)
@@ -1122,7 +1114,9 @@ class Indexer(object):
               typ = self.typemap.get(defn.id)
               if typ:
                 for x in PytypeValue.from_data(typ):
-                  if isinstance(x, Remote):
+                  if x is None:
+                    continue  # Not-yet-special-cased type, e.g. namedtuple.
+                  elif isinstance(x, Remote):
                     links.append((r, x.attr(attr_name)))
                   elif x.typ == "Class":
                     d = self._lookup_class_attr(x.name, attr_name)
@@ -1224,7 +1218,7 @@ class VmTrace(source.AbstractTrace):
     types_repr = tuple(
         t and [node_utils.typename(x) for x in t]
         for t in self.types)
-    return "%s %s" % (super(VmTrace, self).__repr__(), types_repr)
+    return "%s %s" % (super().__repr__(), types_repr)
 
 
 def process_file(options, source_text=None, generate_callgraphs=False,
